﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Shapes;
using System.IO;
using WMPLib;

namespace GNG1103_Launcher
{
    /// <summary>
    /// Interaction logic for Video_Config.xaml
    /// </summary>
    public partial class Video_Config : Window
    {

        #region Directories
        //--------------------------------------------------------DIRECTORIES--------------------------------------------------------------------------
        public string mainDirectory { get; set; } = (string)Application.Current.Properties["ProjectDirectory"];
        public string tempPlaylistDirectory { get; set; } = (string)Application.Current.Properties["ProjectDirectory"] + "\\Playlists\\TempPlaylist.txt";
        public List<string> videosDirectory = new List<string>();
        //---------------------------------------------------------------------------------------------------------------------------------------------
        #endregion

        public Video_Config()
        {
            InitializeComponent();
            this.DataContext = new Directory.ViewModels.ConfigViewModel();
            videosDirectory.Clear();
            getVidDirs();
            
        }

        private void Window_Loaded(object sender, RoutedEventArgs e)
        {
            update();
        }

        private void getVidDirs()
        {
            var lines = File.ReadAllLines(mainDirectory + "\\Directories.txt");
            
            foreach(var line in lines)
            {
                if (line.Split('|')[0].Contains("viDir") == true)
                    videosDirectory.Add(line.Split('|')[1]);
            }
        }

        #region Helper Functions
        //---------------------------------------UPDATE FUNCTIONS----------------------------------------
        void update()
        {
            updatePanel();
            if (VersionDropDown.SelectedIndex == -1)
                VersionDropDown.SelectedIndex = 0;
            if (VersionDropDown.SelectedItem != null)
            {
                updateFile();
            }

        }
        void updatePanel()
        {

                drawPanel();
                var vid = setConfigFile(setVideoFileText());
                if (vid != null)
                {
                    setParams(vid);
                    updateRunTime();
                }

                else
                    wipePanel();


        }

        void updateFile()
        {
            if (VideoSelector.SelectedItem != null)
            {
                if (VersionDropDown.SelectedIndex == 0)
                    goto index0;
                else
                    goto indexNot0;
                index0: if (File.Exists(((Directory.ViewModels.DirectoryItemViewModel)VideoSelector.SelectedItem).FullPath.Split('.')[((Directory.ViewModels.DirectoryItemViewModel)VideoSelector.SelectedItem).FullPath.Split('.').Length - 2] + ".txt") == true)
                    goto exists;
                else
                    goto notExist;
                indexNot0: if (File.Exists(((Directory.ViewModels.DirectoryItemViewModel)VideoSelector.SelectedItem).FullPath.Split('.')[((Directory.ViewModels.DirectoryItemViewModel)VideoSelector.SelectedItem).FullPath.Split('.').Length - 2] + ".txt") == true)
                    goto exists;

                exists: string configPath;
                if (VersionDropDown.SelectedIndex == 0 || VersionDropDown.SelectedItem == null)
                    configPath = configText.Text;
                else
                    configPath = (((Directory.ViewModels.DirectoryItemViewModel)VideoSelector.SelectedItem).FullPath.Split('.')[((Directory.ViewModels.DirectoryItemViewModel)VideoSelector.SelectedItem).FullPath.Split('.').Length - 2] + ";" + VersionDropDown.SelectedItem).Split('.')[0] + ".txt";

                string pM = projectMode.SelectedIndex.ToString();
                string fSS = "0";
                string fSD = "0";
                string cE = CrossEyeText.Text.ToString();
                string vP = pitchText.Text.ToString();
                string vR = RollText.Text.ToString();
                string vY = YawText.Text.ToString();

                try
                {
                    using (var fs = new FileStream(configPath, FileMode.Create, FileAccess.Write, FileShare.ReadWrite))
                    using (var sW = new StreamWriter(fs))
                        sW.Write(pM + ',' + fSS + ',' + fSD + ',' + cE + ',' + vP + "," + vR + "," + vY);
                }catch(IOException)
                {
                    return;
                }
                goto done;

            notExist: if (VersionDropDown.SelectedIndex == 0)
                {
                    var stream = File.Create(((Directory.ViewModels.DirectoryItemViewModel)VideoSelector.SelectedItem).FullPath.Split('.')[((Directory.ViewModels.DirectoryItemViewModel)VideoSelector.SelectedItem).FullPath.Split('.').Length - 2] + ".txt");
                    stream.Close();
                    File.WriteAllText(((Directory.ViewModels.DirectoryItemViewModel)VideoSelector.SelectedItem).FullPath.Split('.')[((Directory.ViewModels.DirectoryItemViewModel)VideoSelector.SelectedItem).FullPath.Split('.').Length - 2] + ".txt", "-1,0,0,0,0,0,0");
                    updateFile();
                }
                else
                {
                    var stream = File.Create(((Directory.ViewModels.DirectoryItemViewModel)VideoSelector.SelectedItem).FullPath.Split(';')[0] + ";" + VersionDropDown.SelectedItem + ".txt");
                    stream.Close();
                    File.WriteAllText(((Directory.ViewModels.DirectoryItemViewModel)VideoSelector.SelectedItem).FullPath.Split(';')[0] + ";" + VersionDropDown.SelectedItem + ".txt", "-1,0,0,0,0,0,0");
                    updateFile();
                }

            done: return;
            }
        }
        //--------------------------------------------------------------------------------------------------

        //------------------------------------------UPDATES-------------------------------------------------
        void updateRunTime()
        {
            var file = fileNameText.Text;

            double time = Duration(file);

            int minutes = (int)time / 60;
            int seconds = (int)time - minutes * 60;
            int hours = minutes / 60;
            minutes = minutes - hours * 60;
            if (seconds < 10)
            {
                if (minutes < 10)
                    playTime.Text = "0" + hours.ToString() + ":0" + minutes.ToString() + ":0" + seconds.ToString();
                if (minutes >= 10)
                    playTime.Text = "0" + hours.ToString() + ":" + minutes.ToString() + ":0" + seconds.ToString();
            }
            if (seconds >= 10)
            {
                if (minutes < 10)
                    playTime.Text = "0" + hours.ToString() + ":0" + minutes.ToString() + ":" + seconds.ToString();
                if (minutes >= 10)
                    playTime.Text = "0" + hours.ToString() + ":" + minutes.ToString() + ":" + seconds.ToString();
            }
        }


        public Double Duration(String file)
        {
            WindowsMediaPlayer wmp = new WindowsMediaPlayer();
            IWMPMedia mediainfo = wmp.newMedia(file);
            return mediainfo.duration;
        }


        void wipePanel()
        {
            configText.Text = "NO CONFIG";
            projectMode.SelectedIndex = -1;
            playTime.Text = "00:00:00";
            CrossEyeText.Text = "0.00";
        }

        void setParams(string v)
        {
            if (VersionDropDown.SelectedItem != null && v != null)
            {
                if (VersionDropDown.SelectedIndex != 0)
                {
                    if (File.Exists(((Directory.ViewModels.DirectoryItemViewModel)VideoSelector.SelectedItem).FullPath.Split('.')[((Directory.ViewModels.DirectoryItemViewModel)VideoSelector.SelectedItem).FullPath.Split('.').Length - 2] + ";" + VersionDropDown.SelectedItem + ".txt") == true)
                    {
                        v = v.Split('.')[v.Split('.').Length - 2] + ".txt";
                        var line = File.ReadAllText(v);
                        var tokens = line.Split(',');

                        projectMode.SelectedIndex = int.Parse(tokens[0]);
                        CrossEyeText.Text = (Convert.ToDouble(tokens[3])).ToString();
                        pitchText.Text = (Convert.ToDouble(tokens[4])).ToString();
                        RollText.Text = (Convert.ToDouble(tokens[5])).ToString();
                        YawText.Text = (Convert.ToDouble(tokens[6])).ToString();
                    }
                    else
                        wipePanel();
                }
                else
                {
                    if (File.Exists(((Directory.ViewModels.DirectoryItemViewModel)VideoSelector.SelectedItem).FullPath.Split('.')[((Directory.ViewModels.DirectoryItemViewModel)VideoSelector.SelectedItem).FullPath.Split('.').Length - 2] + ".txt") == true)
                    {
                        v = v.Split('.')[v.Split('.').Length - 2] + ".txt";
                        var line = File.ReadAllText(v);
                        var tokens = line.Split(',');

                        projectMode.SelectedIndex = int.Parse(tokens[0]);
                        CrossEyeText.Text = (Convert.ToDouble(tokens[3])).ToString();
                        pitchText.Text = (Convert.ToDouble(tokens[4])).ToString();
                        RollText.Text = (Convert.ToDouble(tokens[5])).ToString();
                        YawText.Text = (Convert.ToDouble(tokens[6])).ToString();
                    }
                    else
                        wipePanel();
                }
            }
        }
        void drawPanel()
        {
            List<string> files = new List<string>();
            projMode.Foreground = Brushes.White;
            bool noProjMode = false;
            int set;
            for (int dirIX = 0; dirIX < videosDirectory.Count(); dirIX++)
            {
                files.AddRange(System.IO.Directory.GetFiles(videosDirectory[dirIX]));
            }
            for (int i = 0; i < VideoSelector.Items.Count; i++)
            {
                var videoVersion = new List<string>();
                string baseName = ((Directory.ViewModels.DirectoryItemViewModel)VideoSelector.Items[i]).Name.ToString().Split('.')[((Directory.ViewModels.DirectoryItemViewModel)VideoSelector.Items[i]).Name.ToString().Split('.').Length - 2];
                var item = (Directory.ViewModels.DirectoryItemViewModel)VideoSelector.Items[i];


                if (System.IO.File.Exists(item.FullPath.Split('.')[item.FullPath.Split('.').Length - 2] + ".txt") == true)
                {
                    item.Color = Brushes.Lime;
                    item.Contour = Brushes.Lime;
                    item.Type = Directory.Data.DirectoryItemType.Check;
                }
                else
                {
                    item.Color = Brushes.Red;
                    item.Contour = Brushes.Red;
                    item.Type = Directory.Data.DirectoryItemType.Uncheck;
                }

                foreach (var video in files)
                {
                    if (video.Contains(".mp4") == true)
                    {
                        if (video.Contains(baseName) == true)
                        {
                            if (video.Contains(';') == true)
                                videoVersion.Add(video.Split('\\')[video.Split('\\').Length - 1].Split(';')[1].Split('.')[0]);
                        }
                    }
                }
                if (!item.FullPath.Contains(';'))
                {
                    if (File.Exists(item.FullPath.Split('.')[item.FullPath.Split('.').Length - 2] + ".txt") == true)
                    {
                        set = Convert.ToInt32(File.ReadAllText(item.FullPath.Split('.')[item.FullPath.Split('.').Length - 2] + ".txt").Split(',')[0]);
                        if (set < 1)
                        {
                            item.Color = Brushes.Yellow;
                            item.Contour = Brushes.Yellow;
                            item.Type = Directory.Data.DirectoryItemType.Uncheck;
                            if (item == ((Directory.ViewModels.DirectoryItemViewModel)VideoSelector.SelectedItem))
                                noProjMode = true;
                        }
                    }
                }

                if (item.FullPath.Contains(baseName))
                {
                    for (int j = 0; j < videoVersion.Count; j++)
                    {
                        if (File.Exists(item.FullPath.Split('.')[item.FullPath.Split('.').Length - 2] + ";" + videoVersion[j] + ".txt"))
                        {
                            set = Convert.ToInt32(File.ReadAllText(item.FullPath.Split('.')[item.FullPath.Split('.').Length - 2] + ";" + videoVersion[j] + ".txt").Split(',')[0]);
                            if (set < 1)
                            {
                                // if (VersionDropDown.HasItems == true)
                                //(ComboBoxItem)VersionDropDown.Items[j]. = Brushes.Red;
                                item.Color = Brushes.Yellow;
                                item.Contour = Brushes.Yellow;
                                item.Type = Directory.Data.DirectoryItemType.Uncheck;
                                noProjMode = true;
                            }
                        }
                        else
                        {
                            item.Color = Brushes.Red;
                            item.Contour = Brushes.Red;
                            item.Type = Directory.Data.DirectoryItemType.Uncheck;
                            noProjMode = true;
                        }
                        if (item.FullPath.Contains(baseName) && item.FullPath.Contains(';'))
                        {
                            if (File.Exists(item.FullPath.Split('.')[0] + ";" + videoVersion[j] + ".txt") == false)
                            {

                                item.Color = Brushes.Yellow;
                                item.Contour = Brushes.Yellow;
                                item.Type = Directory.Data.DirectoryItemType.Uncheck;
                            }
                        }
                    }
                }

                if (item == (Directory.ViewModels.DirectoryItemViewModel)VideoSelector.SelectedItem)
                {
                    item.Color = Brushes.DarkTurquoise;
                    if (noProjMode == true)
                        projMode.Foreground = Brushes.Red;
                }
                
            }
        }

       void populateVersionBox(string baseName)
        {
            for (int dirIX = 0; dirIX < videosDirectory.Count(); dirIX++)
            {
                if (((Directory.ViewModels.DirectoryItemViewModel)VideoSelector.SelectedItem).FullPath.Contains(videosDirectory[dirIX]) == true)
                {
                    VersionDropDown.Items.Clear();
                    var files = System.IO.Directory.GetFiles(videosDirectory[dirIX]);
                    string videoVersion;

                    foreach (var video in files)
                    {
                        if (video.Contains(".mp4") == true)
                        {
                            if (video.Contains(baseName + ".") == true || video.Contains(baseName + ";"))
                            {
                                if (video.Contains(';') == true)
                                    VersionDropDown.Items.Add(video.Split('\\')[video.Split('\\').Length - 1].Split(';')[1].Split('.')[video.Split('\\')[video.Split('\\').Length - 1].Split(';')[1].Split('.').Length - 2]);
                                else
                                    VersionDropDown.Items.Add("Default: English");
                            }
                        }
                    }
                }
            }
            versionCountText.Text = VersionDropDown.Items.Count.ToString();
        }

        string setVideoFileText()
        {
            for (int dirIX = 0; dirIX < videosDirectory.Count(); dirIX++)
            {
                if (VideoSelector.SelectedItem != null)
                {
                    if (((Directory.ViewModels.DirectoryItemViewModel)VideoSelector.SelectedItem).FullPath.Contains(videosDirectory[dirIX] + $"\\" + ((Directory.ViewModels.DirectoryItemViewModel)VideoSelector.SelectedItem).Name) == true)
                    {
                        string videoFile;
                        if (VersionDropDown.SelectedItem == null)
                        {
                            fileNameText.Text = "NULL";
                            return null;
                        }
                        if (VersionDropDown.SelectedIndex == 0)
                            videoFile = videosDirectory[dirIX] + "\\" + ((Directory.ViewModels.DirectoryItemViewModel)VideoSelector.SelectedItem).Name.ToString();
                        else
                            videoFile = videosDirectory[dirIX] + "\\" + ((Directory.ViewModels.DirectoryItemViewModel)VideoSelector.SelectedItem).Name.ToString().Split('.')[((Directory.ViewModels.DirectoryItemViewModel)VideoSelector.SelectedItem).Name.ToString().Split('.').Length - 2] + ";" + VersionDropDown.SelectedItem + ".mp4";

                        if (System.IO.File.Exists(videoFile) == true)
                        {
                            fileNameText.Text = videoFile;
                            return videoFile;
                        }
                        else
                        {
                            //System.IO.File.Create(videoFile);
                            fileNameText.Text = videoFile;
                            return videoFile;
                        }
                    }
                }
            }
            return null;
        }

        string setConfigFile(string videoName)
        {
            if (videoName != null)
            {
                if (File.Exists(videoName.Split('.')[videoName.Split('.').Length - 2] + ".txt") == true)
                {
                    configText.Text = videoName.Split('.')[videoName.Split('.').Length - 2] + ".txt";
                }
                else
                {
                    configText.Text = "NO CONFIG";
                }
            }
            return videoName;
        }
        //----------------------------------------------------------------------------------------------------------------------------------


        #endregion

        //----------------------------------------------------CALL UPDATE--------------------------------------------------------------------
        private void VersionDropDown_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
                if (VideoSelector.SelectedItem != null)
                {
                    int sel = VersionDropDown.SelectedIndex;

                    switch (sel)
                    {
                        case 0:
                            {
                                if (File.Exists(((Directory.ViewModels.DirectoryItemViewModel)VideoSelector.SelectedItem).FullPath.Split('.')[0] + ".txt") == true)
                                {
                                    updatePanel();
                                    updateFile();
                                }
                                else
                                {
                                    updateFile();
                                    wipePanel();
                                    updatePanel();
                                }
                            }
                            break;
                        default:
                            {
                                if (File.Exists(((Directory.ViewModels.DirectoryItemViewModel)VideoSelector.SelectedItem).FullPath.Split('.')[0] + ";" + VersionDropDown.SelectedItem + ".txt") == true)
                                {
                                    updatePanel();
                                    updateFile();
                                }
                                else
                                {
                                    updateFile();
                                    wipePanel();
                                    updatePanel();
                                }
                            }
                            break;
                    }
                    update();
                }
            
        }

        private void VideoSelector_SelectedItemChanged(object sender, RoutedPropertyChangedEventArgs<object> e)
        {
            var videoName = ((Directory.ViewModels.DirectoryItemViewModel)VideoSelector.SelectedItem).Name;
            videoName = videoName.Split('.')[0];
            if (VideoSelector.SelectedItem == null)
            {
                populateVersionBox(videoName);
                update();
            }
            else
            {
                populateVersionBox(videoName);
                updatePanel();
            }

        }

        private void ProjectMode_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            if (VideoSelector.SelectedItem != null)
            {
                updateFile();
                drawPanel();
            }
            else
            {
                projectMode.SelectedIndex = -1;
            }
        }
   
        private void EyeInc_Click(object sender, RoutedEventArgs e)
        {
            CrossEyeText.Text = (Convert.ToDouble(CrossEyeText.Text.ToString()) + 0.01).ToString();
        }

        private void EyeDec_Click(object sender, RoutedEventArgs e)
        {
            CrossEyeText.Text = (Convert.ToDouble(CrossEyeText.Text.ToString()) - 0.01).ToString();
        }

        private void CrossEyeText_TextChanged(object sender, TextChangedEventArgs e)
        {
            updateFile();
        }

        private void PitchText_TextChanged(object sender, TextChangedEventArgs e)
        {
            updateFile();
        }

        private void RollText_TextChanged(object sender, TextChangedEventArgs e)
        {
            updateFile();
        }

        private void YawText_TextChanged(object sender, TextChangedEventArgs e)
        {
            updateFile();
        }

        private void PitchDec_Click(object sender, RoutedEventArgs e)
        {
            pitchText.Text = ((Convert.ToDouble(pitchText.Text.ToString())) - 0.5).ToString();
        }

        private void PitchInc_Click(object sender, RoutedEventArgs e)
        {
            pitchText.Text = ((Convert.ToDouble(pitchText.Text.ToString())) + 0.5).ToString();
        }

        private void RollDec_Click(object sender, RoutedEventArgs e)
        {
            RollText.Text = ((Convert.ToDouble(RollText.Text.ToString())) - 0.5).ToString();
        }

        private void RollInc_Click(object sender, RoutedEventArgs e)
        {
            RollText.Text = ((Convert.ToDouble(RollText.Text.ToString())) + 0.5).ToString();
        }

        private void YawDec_Click(object sender, RoutedEventArgs e)
        {
            YawText.Text = ((Convert.ToDouble(YawText.Text.ToString())) - 0.5).ToString();
        }

        private void YawInc_Click(object sender, RoutedEventArgs e)
        {
            YawText.Text = ((Convert.ToDouble(YawText.Text.ToString())) + 0.5).ToString();
        }
        //-----------------------------------------------------------------------------------------------------------------------------------
    }
}
